==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ronportal/system/reportserver.php <==
<?php
/**
 * stanislav.elbl, 23.02.2021
 */

namespace RonPortal\System;
use Exception;
use Ron\Http\HttpResponse;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalSettings;
use RonPortal\User;

/**
 * Zajistuje komunikaci s report serverem
 * Class ReportServer
 */
class ReportServer
{
    private $socket;

    private $errorReporting;
    // pri open si zapamatuje nastaveni error reporting a pri close ho vrati zpet

    private $useRemoteExecutor;

    private $resultData;

    public function __construct()
    {

    }


    public static function getReportSessionId()
    {
        return sha1(session_id());
    }

    /**
     * @param $server
     * @param $port
     * @throws Exception
     */
    public function open($server, $port)
    {
        $this ->errorReporting = error_reporting(E_ERROR); // vypne warningy

        $errno = 0;
        $errstr = '';
        $this ->socket = fsockopen($server, $port, $errno, $errstr);

        if (!$this ->socket) {
            $errstr = StringUtils::fromUTF8($errstr);
            throw new Exception("Nelze navazat spojeni s report serverem!\nChyba cislo $errno: $errstr");
        }
    }


    public function close()
    {
        if ($this ->socket) {
            fclose($this->socket);
            $this->socket = null;
        }

        error_reporting($this ->errorReporting); // nastavi puvodni error reporting
    }


    /**
     * Odesle zpravu, na kterou je odpoved jen OK/Err. Pri chybe vyhodi vyjimku.
     * @param string $msg
     * @throws Exception
     */
    public function sendMessage($msg)
    {
        $msg = StringUtils::toUTF8($msg);

        fputs($this ->socket, $msg . "\n");
        $answer = trim(fgets($this ->socket));

        //stream_set_timeout()

        if ($answer === 'ERR') {
            $answer = StringUtils::fromUTF8(trim(fgets($this ->socket)));

            throw new Exception($answer);
        } else if ($answer !== 'OK') {
            throw new Exception("Chyba pri odesilani prikazu: $msg!");
        }
    }


    /**
     * @param string $msg
     * @param bool $oneRowResult   Pokud je true, nacte se jen jeden radek odpovedi
     * @return string
     * @throws Exception
     */
    public function sendRequest($msg, $oneRowResult = false)
    {
        $msg = StringUtils::toUTF8($msg);
        fputs($this ->socket, $msg . "\n");

        $result = '';
        $xml = null;

        while (1) {
            $answer = fgets($this ->socket);

            if ($answer === false) {
                MessageManager::addDebugLog($answer);
                $info = stream_get_meta_data($this ->socket);
                if ($info &&  $info['timed_out']) {
                    throw new Exception('Chyba pri komunikaci s printserverem - vyprsel casovy limit odpovedi printserveru!');
                }
                throw new Exception('Chyba pri komunikaci s printserverem - prijata neuplna odpoved!');
            }

            $answer = trim($answer);

            if ($oneRowResult) {
                return StringUtils::fromUTF8($answer);
            }

            if (!isset($xml)  &&  $answer === 'XML') {
                $xml = true; // priznak, ze se jedna o XML
            } else if (!$xml  &&  $answer === 'ERR') {
                $answer = StringUtils::fromUTF8(trim(fgets($this ->socket)));

                throw new Exception($answer);
            } else if (!$xml  &&  $answer === 'OK') {
                return StringUtils::fromUTF8($result);
            } else if ($xml  &&  $answer === 'XMLEND') {
                $this ->resultData = $result;
                return $result; // ponecha v UTF8!
            } else {
                $result .= $answer . "\n";
            }

            if (!isset($xml)) {
                $xml = false;
            }
        }

        return ''; // tu se vubec nedostane
    }


    /**
     * @throws Exception
     */
    public function sendXmlRemoteExecutor($xml)
    {
        global $reportserver;
        global $reportserverport;

        $agentManager = new AgentManager();
        $agentResult = $agentManager->executeAgentRemote($reportserver, $reportserverport,
            'ADS_Agent_PrintServer.exe',
            '-master -MasterExport', true, false, $xml);

        if ($agentResult && $agentResult ->agentResult) {
            if ($agentResult ->logText) {
                //  . ($finalmessage ? ' ' . $finalmessage : '') ); - finalmessage nepridavat, aby se nezobrazovaly hlasky dvojmo!
                throw new Exception($agentResult ->logText);
            }

            throw new Exception( 'Agent byl ukoncen chybou: kod ' . $agentResult ->agentResult . ($agentResult ->finalMessage ? ' ' . $agentResult ->finalMessage : '') );
        }

        if (isset($agentResult ->resultParams['resultdata'])) {
            $this ->resultData = $agentResult ->resultParams['resultdata'];
            return $this ->resultData;
        }

        return null;
    }


    /**
     * @throws Exception
     */
    public function sendXmlRequest($xmlContent)
    {
        $time = date('j.n.Y G:i:s', time());
        $pwd = 'i7xU-D91AhjM/prD6'; // TODO: umoznit uzivatelske nastaveni hesla
        $xml =
            "<?xml version='1.0'?>\n" .  //  encoding='utf-8'
            "<report userRC='" . StringUtils::xmlTextConvert(User::getRC()) . "' key='" . sha1($pwd . StringUtils::toUTF8(User::getRC()) . $time) . "' time='$time'>\n" .
            $xmlContent .
            "</report>\n";

        if ($this ->useRemoteExecutor) {
            return $this ->sendXmlRemoteExecutor($xml);
        }

        return $this->sendRequest("XML\n" . $xml . "\nXMLEND", false);
    }


    /**
     * Zapise na vystup zbyvajici obsah streamu
     */
    public function outputContent()
    {
        if ($this ->useRemoteExecutor) {
            echo $this->resultData;
        } else {
            fpassthru($this->socket);
        }
    }


    public function downloadContent()
    {
        if ($this ->useRemoteExecutor) {
            return $this ->resultData;
        }

        return stream_get_contents($this ->socket);
    }

    /**
     * Spusteni skriptu zadaneho jmena
     * @param $category
     * @param $name
     * @param $scriptParam
     * @throws Exception
     */
    public function executeScript($category, $name, $scriptParam) {
        $this ->sendMessage('USER ' . User::getLoginName() . "\n");
        $this ->sendMessage('LONGVAR AgentParameter=' . $scriptParam . "\n");

        $this ->sendMessage("SCRIPT $category/$name\n");

        $this ->sendMessage('EXIT');
    }


//    /**
//     * Vygenerovani reportu pro stare sestavy (ne z manazera sestav)
//     * @param string $reportName
//     * @param string $format
//     * @param PracovniPomer $pp
//     * @param string[]  $stringParams
//     * @param string[]  $dateParams
//     * @param int[]     $integerParams
//     * @throws \Exception
//     */
//    function createReport($reportName, $format, $pp, $stringParams, $dateParams, $integerParams)
//    {
//        $this ->sendMessage('USER ' . User::getLoginName() . "\n");
//
//        $this ->sendMessage('VAR LoginName=' . User::getLoginName() . "\n");
//        $this ->sendMessage('VAR RealLoginName=' . User::getRealLoginName() . "\n");
//        if ($pp) {
//            $this ->sendMessage(
//                'VAR RC=' . $pp->rc .
//                ';osobniCislo=' . $pp->osobniCislo .
//                "\n");
//
//            $this ->sendMessage('DATEVAR datumNastupu=' . date('j.n.Y', $pp->datumNastupu) . "\n");
//        } else {
//            $this->sendMessage("VAR RC=\n");
//        }
//
//        // odesle dalsi prikazy (napr. parametry - id sluzebni cesty a podobne)
//        $paramsText = '';
//        if ($stringParams) {
//            foreach ($stringParams as $name => $value) {
//                $paramsText .= ($paramsText ? ';' : '') . $name . '=' . $value;
//            }
//        }
//        // stare sestavy (ne z manazera) pouzivaly misto integeru stringy - prida je ke stringum
//        foreach ($integerParams as $name => $value) {
//            $paramsText .= ($paramsText ? ';' : '') . $name . '=' . $value;
//        }
//
//        if ($paramsText) {
//            $this ->sendMessage("VAR $paramsText\n");
//        }
//
//        if ($dateParams) {
//            $paramsText = '';
//            foreach ($dateParams as $name => $value) {
//                $paramsText .= ($paramsText ? ';' : '') . $name . '=' . $value;
//            }
//            $this ->sendMessage("DATEVAR $paramsText\n");
//        }
//
//        if ($format) {
//            $this ->sendMessage("FORMAT $format\n");
//        }
//
//        //$this ->sendMessage( iconv('windows-1250', 'utf-8', "PRINT $reportName\n") );  // 22.12.2016 - standa - patri tu iconv? pro sestavy s diakritikou to nefachci!
//        $this ->sendMessage( "PRINT $reportName\n" );
//    }


    public static function getFileExtension($format)
    {
        if (!$format) {
            return 'pdf';
        }

        switch (strtolower($format)) {
            case 'pdf' :
            case 'pdfa' :
                return 'pdf';

            default :
                return $format;
        }
    }

//    /**
//     * Vyexportovani reportu pro stare sestavy
//     * @param string $reportName
//     * @param string $format
//     * @param PracovniPomer $pp
//     * @param string[] $stringParams
//     * @param string[] $dateParams
//     * @param int[] $integerParams
//     * @throws \Exception
//     */
//    function outputReport($reportName, $format, $pp, $stringParams, $dateParams, $integerParams)
//    {
//        $this ->createReport($reportName, $format, $pp, $stringParams, $dateParams, $integerParams);
//
//        HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent($reportName . '.' . self::getFileExtension($format), 0);
//        HttpResponse::getInstance() ->renderHeaders();
//
//        $this ->outputContent();
//    }


//    /**
//     * Nacte informace o stavu reportServeru
//     * @return string
//     * @throws Exception
//     */
//    function getInfo()
//    {
//        $info = $this ->sendRequest('DBINFO');
//        $this ->sendMessage('EXIT');
//
//        return $info;
//    }

    /**
     * @throws Exception
     */
    public static function getServerInfo()
    {
        $xmlContent =
            "  <serverinfo/>\n";

        $server = self::createInstance();
        try {
            $xml = $server->sendXmlRequest($xmlContent);

            return XmlLoader::loadXmlToArray($xml, [XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true, XmlLoader::PARAM_CONVERT_FROM_UTF8 => true], 'utf-8');
        } catch (Exception $e) {
            $server->close();
            throw $e;
        }
    }


    /**
     * Nacte informace o dane sestave z ccf
     * @param Report $report
     * @return array
     * @throws Exception
     */
    public static function getReportInfo($report)
    {
        $reportNameXml = StringUtils::xmlTextConvert($report->kod);

        $xmlContent =
            "  <reportinfo kod='$reportNameXml' />\n";

        $server = self::createInstance();
        try {
            $xml = $server->sendXmlRequest($xmlContent);
            return XmlLoader::loadXmlToArray($xml, [XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true, XmlLoader::PARAM_CONVERT_FROM_UTF8 => true]);
        } catch (Exception $e) {
            $server->close();
            throw $e;
        }
    }


    /**
     * Zpracuje informace o sestave nactene z ccf pomoci reportserveru
     * @param Report $report
     * @param array $info
     */
    public static function processReportInfo($report, $info)
    {
        // zjisti z printserveru aktivni panely ...
        $report->zobrazitPanelObdobi = true;
        $report->zobrazitPanelOsoba = true;
        $report->zobrazitPanelKalendar = true;
        $report->zobrazitPanelRozdeleni = true;
        $report->zobrazitPanelRozdeleni2 = true;
        $report->zobrazitPanelRazeni = true;

        if ( $info['panels'] instanceof \ArrayAccess &&  $info['panels'][0]['panel'] instanceof \ArrayAccess ) {
            //var_dump($info['panels'][0]['panel']);
            foreach ($info['panels'][0]['panel'] as $panelInfo) {
                switch (strtolower($panelInfo['name'])) {
                    case 'cp_obdobi' :
                        $report ->zobrazitPanelObdobi = $panelInfo['visible'] != '0';
                        break;
                    case 'cp_osoba' :
                        $report ->zobrazitPanelOsoba = $panelInfo['visible'] != '0';
                        break;
                    case 'cp_kalendar' :
                        $report ->zobrazitPanelKalendar = $panelInfo['visible'] != '0';
                        break;

                    case 'cp_rozdeleni' :
                        $report ->zobrazitPanelRozdeleni = $panelInfo['visible'] != '0';
                        break;
                    case 'cp_rozdeleni2' :
                        $report ->zobrazitPanelRozdeleni2 = $panelInfo['visible'] != '0';
                        break;

                    case 'cp_ordering' :
                        $report ->zobrazitPanelRazeni = $panelInfo['visible'] != '0';
                        break;

                    case 'cp_filter' :
                    case 'cp_grouping' :
                    case 'cp_osoba_vycet' :
                    case 'cp_rozdeleni_v1' :
                    case 'cp_rozdeleni_v2' :
                        break;
                }
            }
        }

        $report ->orderFieldsList = [];
        //var_dump($info['orderfield']);
        if ( $info['orderfield']  && $info['orderfield'] instanceof \ArrayAccess) {
            foreach ($info['orderfield'] as $orderField) {
                // pro nalezeni prekladu odstrani origin (jmeno tabulky)
                $fieldName = strtolower($orderField['fieldname']);
//                $dotPos = strrpos($fieldName, '.');
//                $fieldName = $dotPos === false
//                    ? $fieldName
//                    : substr($fieldName, $dotPos + 1);

                $text = TextManager::getInstance() ->getTranslatedText('report_sort_' . $fieldName, '');
                if ($text) {
                    $report ->orderFieldsList[$orderField['fieldname']] = $text;
                }
            }
        }

        //var_dump($report);
    }


    /**
     * @throws Exception
     */
    public static function loadReportInfo($report)
    {
        $info = self::getReportInfo($report);
        self::processReportInfo($report, $info);
    }


    /**
     * @return ReportServer
     * @throws Exception
     */
    public static function createInstance()
    {
        global $reportserver;
        global $reportserverport;

        if (!$reportserver || !$reportserverport) {
            throw new Exception('Chyba pri spousteni skriptu pomoci reportserveru: v _appdbset.php neni nastaveno spojeni na reportserver!');
        }

        $agentHost = RonPortalSettings::getValue(RonPortalSettings::ADS_AGENT_HOST);
        $agentPort = (int)RonPortalSettings::getValue(RonPortalSettings::ADS_AGENT_PORT);

        $result = new self();
        if ($agentHost === $reportserver  &&  $agentPort === $reportserverport) {
            // pokud je host a port stejny jako remote executor, tak ho pouzije
            $result ->useRemoteExecutor = true;
        } else {
            $result->open($reportserver, $reportserverport);
        }

        return $result;
    }
}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2311_175.zip: ronportal/ronportal/system/reportserver.php <==
<?php
/**
 * stanislav.elbl, 23.02.2021
 */

namespace RonPortal\System;
use Exception;
use Ron\Http\HttpResponse;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalSettings;
use RonPortal\User;
use Tracy\Debugger;

/**
 * Zajistuje komunikaci s report serverem
 * Class ReportServer
 */
class ReportServer
{
    private $socket;

    private $errorReporting;
    // pri open si zapamatuje nastaveni error reporting a pri close ho vrati zpet

    private $useRemoteExecutor;

    private $resultData;

    public function __construct()
    {

    }


    public static function getReportSessionId()
    {
        return sha1(session_id());
    }

    /**
     * @param $server
     * @param $port
     * @throws Exception
     */
    public function open($server, $port)
    {
        $this ->errorReporting = error_reporting(E_ERROR); // vypne warningy

        $errno = 0;
        $errstr = '';
        $this ->socket = fsockopen($server, $port, $errno, $errstr);

        if (!$this ->socket) {
            $errstr = StringUtils::fromUTF8($errstr);
            throw new Exception("Nelze navazat spojeni s report serverem!\nChyba cislo $errno: $errstr");
        }
    }


    public function close()
    {
        if ($this ->socket) {
            fclose($this->socket);
            $this->socket = null;
        }

        error_reporting($this ->errorReporting); // nastavi puvodni error reporting
    }


    /**
     * Odesle zpravu, na kterou je odpoved jen OK/Err. Pri chybe vyhodi vyjimku.
     * @param string $msg
     * @throws Exception
     */
    public function sendMessage($msg)
    {
        $msg = StringUtils::toUTF8($msg);

        fputs($this ->socket, $msg . "\n");
        $answer = trim(fgets($this ->socket));

        //stream_set_timeout()

        if ($answer === 'ERR') {
            $answer = StringUtils::fromUTF8(trim(fgets($this ->socket)));

            throw new Exception($answer);
        } else if ($answer !== 'OK') {
            throw new Exception("Chyba pri odesilani prikazu: $msg!");
        }
    }


    /**
     * @param string $msg
     * @param bool $oneRowResult   Pokud je true, nacte se jen jeden radek odpovedi
     * @return string
     * @throws Exception
     */
    public function sendRequest($msg, $oneRowResult = false)
    {
        $msg = StringUtils::toUTF8($msg);
        fputs($this ->socket, $msg . "\n");

        $result = '';
        $xml = null;

        while (1) {
            $answer = fgets($this ->socket);

            if ($answer === false) {
                MessageManager::addDebugLog($answer);
                $info = stream_get_meta_data($this ->socket);
                if ($info &&  $info['timed_out']) {
                    throw new Exception('Chyba pri komunikaci s printserverem - vyprsel casovy limit odpovedi printserveru!');
                }
                throw new Exception('Chyba pri komunikaci s printserverem - prijata neuplna odpoved!');
            }

            $answer = trim($answer);

            if ($oneRowResult) {
                return StringUtils::fromUTF8($answer);
            }

            if (!isset($xml)  &&  $answer === 'XML') {
                $xml = true; // priznak, ze se jedna o XML
            } else if (!$xml  &&  $answer === 'ERR') {
                $answer = StringUtils::fromUTF8(trim(fgets($this ->socket)));

                throw new Exception($answer);
            } else if (!$xml  &&  $answer === 'OK') {
                return StringUtils::fromUTF8($result);
            } else if ($xml  &&  $answer === 'XMLEND') {
                $this ->resultData = $result;
                return $result; // ponecha v UTF8!
            } else {
                $result .= $answer . "\n";
            }

            if (!isset($xml)) {
                $xml = false;
            }
        }

        return ''; // tu se vubec nedostane
    }


    /**
     * @throws Exception
     */
    public function sendXmlRemoteExecutor($xml, $app = null)
    {
        global $reportserver;

        if (is_array($reportserver)) {
            $settings = $app
                ? $reportserver[strtoupper($app)] ?? null
                : null;

            if (!$settings) {   // pokud neni nastaveni pro konkretni aplikaci, tak pouzije prvni nastaveni
                $app = array_key_first($reportserver);  // Dostupne od PHP 7.3!
                $settings = $reportserver[$app] ?? null;
            }

            if (!$settings) {
                throw new Exception('Spatne nastaveni reportserveru - pole $reportserver neobsahuje zadnou polozku!');
            }

            $server = $settings['server'];
            $port = $settings['port'];
        } else {
            $app = $GLOBALS['reportserverapp'] ?? 'ADS';
            $server = $reportserver;
            $port = $GLOBALS['reportserverport'] ?? 9090;
        }

        $agentManager = new AgentManager();
        $agentResult = $agentManager->executeAgentRemote($server, $port,
            $app . '_Agent_PrintServer.exe',
            '-master -MasterExport', true, false, $xml);

        if ($agentResult && $agentResult ->agentResult) {
            if ($agentResult ->logText) {
                //  . ($finalmessage ? ' ' . $finalmessage : '') ); - finalmessage nepridavat, aby se nezobrazovaly hlasky dvojmo!
                throw new Exception($agentResult ->logText);
            }

            throw new Exception( 'Agent byl ukoncen chybou: kod ' . $agentResult ->agentResult . ($agentResult ->finalMessage ? ' ' . $agentResult ->finalMessage : '') );
        }

        if (isset($agentResult ->resultParams['resultdata'])) {
            $this ->resultData = $agentResult ->resultParams['resultdata'];
            if ($GLOBALS['reportserver_debugdir']) {
                file_put_contents($GLOBALS['reportserver_debugdir'] . '/reportserver_' . date('Ymd-His', time()) . '.xml', $this ->resultData);
            }
            return $this ->resultData;
        }

        return null;
    }


    /**
     * @throws Exception
     */
    public function sendXmlRequest($xmlContent, $app = null)
    {
        $time = date('j.n.Y G:i:s', time());
        $pwd = 'i7xU-D91AhjM/prD6'; // TODO: umoznit uzivatelske nastaveni hesla
        $xml =
            "<?xml version='1.0'?>\n" .  //  encoding='utf-8'
            "<report userRC='" . StringUtils::xmlTextConvert(User::getRC()) . "' key='" . sha1($pwd . StringUtils::toUTF8(User::getRC()) . $time) . "' time='$time'>\n" .
            $xmlContent .
            "</report>\n";

        if ($this ->useRemoteExecutor) {
            return $this ->sendXmlRemoteExecutor($xml, $app);
        }

        return $this->sendRequest("XML\n" . $xml . "\nXMLEND", false);
    }


    /**
     * Zapise na vystup zbyvajici obsah streamu
     */
    public function outputContent()
    {
        if ($this ->useRemoteExecutor) {
            echo $this->resultData;
        } else {
            fpassthru($this->socket);
        }
    }


    public function downloadContent()
    {
        if ($this ->useRemoteExecutor) {
            return $this ->resultData;
        }

        return stream_get_contents($this ->socket);
    }

    /**
     * Spusteni skriptu zadaneho jmena
     * @param $category
     * @param $name
     * @param $scriptParam
     * @throws Exception
     */
    public function executeScript($category, $name, $scriptParam) {
        $this ->sendMessage('USER ' . User::getLoginName() . "\n");
        $this ->sendMessage('LONGVAR AgentParameter=' . $scriptParam . "\n");

        $this ->sendMessage("SCRIPT $category/$name\n");

        $this ->sendMessage('EXIT');
    }


//    /**
//     * Vygenerovani reportu pro stare sestavy (ne z manazera sestav)
//     * @param string $reportName
//     * @param string $format
//     * @param PracovniPomer $pp
//     * @param string[]  $stringParams
//     * @param string[]  $dateParams
//     * @param int[]     $integerParams
//     * @throws \Exception
//     */
//    function createReport($reportName, $format, $pp, $stringParams, $dateParams, $integerParams)
//    {
//        $this ->sendMessage('USER ' . User::getLoginName() . "\n");
//
//        $this ->sendMessage('VAR LoginName=' . User::getLoginName() . "\n");
//        $this ->sendMessage('VAR RealLoginName=' . User::getRealLoginName() . "\n");
//        if ($pp) {
//            $this ->sendMessage(
//                'VAR RC=' . $pp->rc .
//                ';osobniCislo=' . $pp->osobniCislo .
//                "\n");
//
//            $this ->sendMessage('DATEVAR datumNastupu=' . date('j.n.Y', $pp->datumNastupu) . "\n");
//        } else {
//            $this->sendMessage("VAR RC=\n");
//        }
//
//        // odesle dalsi prikazy (napr. parametry - id sluzebni cesty a podobne)
//        $paramsText = '';
//        if ($stringParams) {
//            foreach ($stringParams as $name => $value) {
//                $paramsText .= ($paramsText ? ';' : '') . $name . '=' . $value;
//            }
//        }
//        // stare sestavy (ne z manazera) pouzivaly misto integeru stringy - prida je ke stringum
//        foreach ($integerParams as $name => $value) {
//            $paramsText .= ($paramsText ? ';' : '') . $name . '=' . $value;
//        }
//
//        if ($paramsText) {
//            $this ->sendMessage("VAR $paramsText\n");
//        }
//
//        if ($dateParams) {
//            $paramsText = '';
//            foreach ($dateParams as $name => $value) {
//                $paramsText .= ($paramsText ? ';' : '') . $name . '=' . $value;
//            }
//            $this ->sendMessage("DATEVAR $paramsText\n");
//        }
//
//        if ($format) {
//            $this ->sendMessage("FORMAT $format\n");
//        }
//
//        //$this ->sendMessage( iconv('windows-1250', 'utf-8', "PRINT $reportName\n") );  // 22.12.2016 - standa - patri tu iconv? pro sestavy s diakritikou to nefachci!
//        $this ->sendMessage( "PRINT $reportName\n" );
//    }


    public static function getFileExtension($format)
    {
        if (!$format) {
            return 'pdf';
        }

        switch (strtolower($format)) {
            case 'pdf' :
            case 'pdfa' :
                return 'pdf';

            default :
                return $format;
        }
    }

//    /**
//     * Vyexportovani reportu pro stare sestavy
//     * @param string $reportName
//     * @param string $format
//     * @param PracovniPomer $pp
//     * @param string[] $stringParams
//     * @param string[] $dateParams
//     * @param int[] $integerParams
//     * @throws \Exception
//     */
//    function outputReport($reportName, $format, $pp, $stringParams, $dateParams, $integerParams)
//    {
//        $this ->createReport($reportName, $format, $pp, $stringParams, $dateParams, $integerParams);
//
//        HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent($reportName . '.' . self::getFileExtension($format), 0);
//        HttpResponse::getInstance() ->renderHeaders();
//
//        $this ->outputContent();
//    }


//    /**
//     * Nacte informace o stavu reportServeru
//     * @return string
//     * @throws Exception
//     */
//    function getInfo()
//    {
//        $info = $this ->sendRequest('DBINFO');
//        $this ->sendMessage('EXIT');
//
//        return $info;
//    }

    /**
     * @throws Exception
     */
    public static function getServerInfo()
    {
        $xmlContent =
            "  <serverinfo/>\n";

        $server = self::createInstance();
        try {
            $xml = $server->sendXmlRequest($xmlContent);

            return XmlLoader::loadXmlToArray($xml, [XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true, XmlLoader::PARAM_CONVERT_FROM_UTF8 => true], 'utf-8');
        } catch (Exception $e) {
            $server->close();
            throw $e;
        }
    }


    /**
     * Nacte informace o dane sestave z ccf
     * @param Report $report
     * @return array
     * @throws Exception
     */
    public static function getReportInfo($report)
    {
        $reportNameXml = StringUtils::xmlTextConvert($report->kod);

        $xmlContent =
            "  <reportinfo kod='$reportNameXml' />\n";

        $server = self::createInstance();
        try {
            $xml = $server->sendXmlRequest($xmlContent, $report ->applicationId);
            return XmlLoader::loadXmlToArray($xml, [XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true, XmlLoader::PARAM_CONVERT_FROM_UTF8 => true]);
        } catch (Exception $e) {
            $server->close();
            throw $e;
        }
    }


    /**
     * Zpracuje informace o sestave nactene z ccf pomoci reportserveru
     * @param Report $report
     * @param array $info
     */
    public static function processReportInfo($report, $info)
    {
        // zjisti z printserveru aktivni panely ...
        $report->zobrazitPanelObdobi = true;
        $report->zobrazitPanelOsoba = true;
        $report->zobrazitPanelKalendar = true;
        $report->zobrazitPanelRozdeleni = true;
        $report->zobrazitPanelRozdeleni2 = true;
        $report->zobrazitPanelRazeni = true;

        if ( $info['panels'] instanceof \ArrayAccess &&  $info['panels'][0]['panel'] instanceof \ArrayAccess ) {
            //var_dump($info['panels'][0]['panel']);
            foreach ($info['panels'][0]['panel'] as $panelInfo) {
                switch (strtolower($panelInfo['name'])) {
                    case 'cp_obdobi' :
                        $report ->zobrazitPanelObdobi = $panelInfo['visible'] != '0';
                        break;
                    case 'cp_osoba' :
                        $report ->zobrazitPanelOsoba = $panelInfo['visible'] != '0';
                        break;
                    case 'cp_kalendar' :
                        $report ->zobrazitPanelKalendar = $panelInfo['visible'] != '0';
                        break;

                    case 'cp_rozdeleni' :
                        $report ->zobrazitPanelRozdeleni = $panelInfo['visible'] != '0';
                        break;
                    case 'cp_rozdeleni2' :
                        $report ->zobrazitPanelRozdeleni2 = $panelInfo['visible'] != '0';
                        break;

                    case 'cp_ordering' :
                        $report ->zobrazitPanelRazeni = $panelInfo['visible'] != '0';
                        break;

                    case 'cp_filter' :
                    case 'cp_grouping' :
                    case 'cp_osoba_vycet' :
                    case 'cp_rozdeleni_v1' :
                    case 'cp_rozdeleni_v2' :
                        break;
                }
            }
        }

        $report ->orderFieldsList = [];
        //var_dump($info['orderfield']);
        if ( $info['orderfield']  && $info['orderfield'] instanceof \ArrayAccess) {
            foreach ($info['orderfield'] as $orderField) {
                // pro nalezeni prekladu odstrani origin (jmeno tabulky)
                $fieldName = strtolower($orderField['fieldname']);
//                $dotPos = strrpos($fieldName, '.');
//                $fieldName = $dotPos === false
//                    ? $fieldName
//                    : substr($fieldName, $dotPos + 1);

                $text = TextManager::getInstance() ->getTranslatedText('report_sort_' . $fieldName, '');
                if ($text) {
                    $report ->orderFieldsList[$orderField['fieldname']] = $text;
                }
            }
        }

        //var_dump($report);
    }


    /**
     * @throws Exception
     */
    public static function loadReportInfo($report)
    {
        $info = self::getReportInfo($report);
        self::processReportInfo($report, $info);
    }


    /**
     * @return ReportServer
     * @throws Exception
     */
    public static function createInstance()
    {
        global $reportserver;
        global $reportserverport;
        global $reportserverapp;

        if (!$reportserver) {
            throw new Exception('Chyba pri spousteni skriptu pomoci reportserveru: v _appdbset.php neni nastaveno spojeni na reportserver!');
        }

        //$agentHost = RonPortalSettings::getValue(RonPortalSettings::ADS_AGENT_HOST);
        $agentPort = (int)RonPortalSettings::getValue(RonPortalSettings::ADS_AGENT_PORT);

        $result = new self();
        if ($reportserverapp  ||  $agentPort === $reportserverport  ||  is_array($reportserver)) {
            // pouzije remote executor, pokud je nastaven kod aplikace nebo je port stejny jako remote executor
            // nebo je $reportserver nastaven jako pole (novy zpusob nastaveni umoznujici pouzit pro kazdou aplikaci jineho agenta)
            $result ->useRemoteExecutor = true;
        } else {
            $result->open($reportserver, $reportserverport);
        }

        return $result;
    }
}
